from typing import Dict
from typing import List


def compare_and_get_notification_payload(
    hub, new_notifications: Dict[str, List], old_state: Dict[str, List]
) -> Dict[str, List]:
    """
    This functions helps in comparing two dicts.
    It compares each key value in both the dicts and return merged notifications configuration

    Returns:
        {Resultant notifications dictionary}

    """
    if "notifications" in old_state:
        existing_notifications = old_state["notifications"]
        notifications_result = {}
        for key in new_notifications:
            if not hub.tool.aws.state_comparison_utils.are_lists_identical(
                existing_notifications[key], new_notifications[key]
            ):
                notifications_result[key] = new_notifications[key]

        return notifications_result
    else:
        return new_notifications



import json
from typing import Any
from typing import Dict


def convert_raw_bucket_notification_to_present(
    hub, raw_resource: Dict[str, Any], bucket_name: str
) -> Dict[str, Any]:
    """
    Convert the s3 bucket notification configurations response to a common format

    Args:
        raw_resource: List of s3 bucket notification configurations
        bucket_name: Name of the bucket on which notification needs to be configured.

    Returns:
        A dictionary of s3 bucket notification configurations
    """
    translated_resource = {}
    raw_resource.pop("ResponseMetadata", None)

    if raw_resource:
        translated_resource["name"] = bucket_name
        translated_resource["resource_id"] = bucket_name + "-notifications"
        translated_resource["notifications"] = raw_resource
    return translated_resource


def convert_raw_public_access_block_to_present(
    hub, bucket_name: str, public_access_block_configuration: Dict
) -> Dict[str, Any]:

    translated_resource = {
        "name": f"{bucket_name}-public-access-block",
        "bucket": bucket_name,
        "public_access_block_configuration": json.loads(
            json.dumps(public_access_block_configuration)
        ),
        "resource_id": bucket_name,
    }
    return translated_resource



from collections import OrderedDict
from typing import Any
from typing import Dict
from typing import List

"""
Util functions to convert raw resource state from AWS ECR to present input format.
"""


def convert_raw_repository_to_present(
    hub, raw_resource: Dict[str, Any], tags: List = None, idem_resource_name: str = None
) -> Dict[str, Any]:
    resource_id = raw_resource.get("repositoryName")
    resource_parameters = OrderedDict(
        {
            "repositoryArn": "repository_arn",
            "registryId": "registry_id",
            "repositoryName": "repository_name",
            "repositoryUri": "repository_uri",
            "imageTagMutability": "image_tag_mutability",
            "imageScanningConfiguration": "image_scanning_configuration",
            "encryptionConfiguration": "encryption_configuration",
        }
    )

    resource_translated = {"name": idem_resource_name, "resource_id": resource_id}
    for parameter_raw, parameter_present in resource_parameters.items():
        if parameter_raw in raw_resource and raw_resource.get(parameter_raw):
            resource_translated[parameter_present] = raw_resource.get(parameter_raw)

    if tags:
        resource_translated["tags"] = tags

    return resource_translated



from typing import Dict
from typing import List


def get_hosted_zone_with_filters(
    hub,
    ctx,
    raw_hosted_zones: Dict,
    hosted_zone_name: str = None,
    private_zone: bool = None,
    vpc_id: str = None,
    tags: List = None,
):
    """
    Returns the hosted_zone with the specified filters, if it is available.

    Args:
        raw_hosted_zones(Dict): Dict of all the described hosted_zones
        hosted_zone_name(string, optional): Domain name of hosted_zone to filter.
        private_zone(bool, optional): Bool argument to specify a private hosted_zone. One of the filter option for hosted_zone
        vpc_id(string, optional): The vpc_id associated with the hosted_zone. One of the filter option for hosted_zone
        tags(List, optional): Tags of the hosted_zone. One of the filter option for hosted_zone

    """
    result = dict(comment=(), result=True, ret=None)
    raw_hosted_zones = raw_hosted_zones.values()
    hosted_zones = []
    for hosted_zone in raw_hosted_zones:
        temp = hosted_zone["aws.route53.hosted_zone.present"]
        temp_hosted_zone = {}
        for sub_dict in temp:
            temp_hosted_zone.update(sub_dict)
        hosted_zones.append(temp_hosted_zone)

    # filter_hosted_zones() returns True if all the filters match for a hosted_zone, and it is added to the list
    filtered_hosted_zones = list(
        filter(
            lambda x: filter_hosted_zones(
                x, hosted_zone_name, private_zone, vpc_id, tags
            ),
            hosted_zones,
        )
    )
    if not filtered_hosted_zones:
        result["comment"] = (
            f"Unable to find aws.route53.hosted_zone resource with given filters",
        )
        return result
    if len(filtered_hosted_zones) > 1:
        result["comment"] = (
            f"More than one aws.route53.hosted_zone resource was found with given filters. Use resource {filtered_hosted_zones[0].get('resource_id')}",
        )
    else:
        result["comment"] = (
            f"Found this aws.route53.hosted_zone resource {filtered_hosted_zones[0].get('resource_id')} with given filters",
        )
    result["ret"] = filtered_hosted_zones[0]
    return result


def filter_hosted_zones(
    hosted_zone: Dict,
    hosted_zone_name: str = None,
    private_zone: bool = None,
    vpc_id: str = None,
    tags: List = None,
):
    """
    Returns True if the hosted_zone checks all the filters provided or return False

    Args:
        hosted_zone(Dict): The described hosted_zone
        hosted_zone_name(string, optional): Domain name of hosted_zone to filter.
        private_zone(bool, optional): Bool argument to specify a private hosted_zone. One of the filter option for hosted_zone
        vpc_id(string, optional): The vpc_id associated with the hosted_zone. One of the filter option for hosted_zone
        tags(List, optional): Tags of the hosted_zone. One of the filter option for hosted_zone

    """
    # Return True if all the provided filters match or return False.

    if hosted_zone_name:
        if hosted_zone["hosted_zone_name"] != hosted_zone_name:
            return False

    if private_zone is not None:
        if hosted_zone["config"]["PrivateZone"] != private_zone:
            return False

    if vpc_id:
        found = False
        if hosted_zone["vpcs"]:
            for vpc in hosted_zone["vpcs"]:
                if vpc["VPCId"] == vpc_id:
                    found = True
                    break
            if not found:
                return False

    # Checking if all the tags in the filter match with the tags present in the hosted_zone.If not we return False
    if tags:
        tags2 = hosted_zone.get("tags")
        if tags2 is None:
            return False
        tags2_map = {tag.get("Key"): tag for tag in tags2}
        for tag in tags:
            if tag["Key"] not in tags2_map or (
                tags2_map.get(tag["Key"]).get("Value") != tag["Value"]
            ):
                return False

    return True



from collections import OrderedDict
from typing import Any
from typing import Dict


def convert_raw_monitor_to_present(
    hub, ctx, raw_resource: Dict[str, Any], idem_resource_name: str
) -> Dict[str, Any]:
    resource_id = raw_resource.get("MonitorArn")
    resource_parameters = OrderedDict(
        {
            "MonitorSpecification": "monitor_specification",
            "MonitorType": "monitor_type",
            "MonitorName": "monitor_name",
        }
    )

    resource_translated = {
        "name": idem_resource_name,
        "resource_id": resource_id,
    }
    for parameter_raw, parameter_present in resource_parameters.items():
        if parameter_raw in raw_resource:
            resource_translated[parameter_present] = raw_resource.get(parameter_raw)

    return resource_translated


def convert_raw_subscription_to_present(
    hub, ctx, raw_resource: Dict[str, Any], idem_resource_name: str
) -> Dict[str, Any]:
    resource_id = raw_resource.get("SubscriptionArn")
    resource_parameters = OrderedDict(
        {
            "MonitorArnList": "monitor_arn_list",
            "Subscribers": "subscribers",
            "Threshold": "threshold",
            "Frequency": "frequency",
            "SubscriptionName": "subscription_name",
        }
    )

    resource_translated = {"name": idem_resource_name, "resource_id": resource_id}
    for parameter_raw, parameter_present in resource_parameters.items():
        if parameter_raw in raw_resource:
            resource_translated[parameter_present] = raw_resource.get(parameter_raw)

    return resource_translated
